@@ -1,3 +1,5 @@
+
+// We don't want this freed by the delete function.
 #include <map>
 #include "Strings.h"
 #include "TodoDB.h"
@@ -8,10 +10,15 @@
 using namespace term;
 using namespace str;
 
+static bool coloursInitialised = false;
+
 map<string, TodoDB::StreamColour> TodoDB::streamColour;
 
 TodoDB::TodoDB() {
-	initColour();
+	if (!coloursInitialised) {
+		initColour();
+		coloursInitialised = true;
+	}
 }
 
 TodoDB::TodoDB(string const &file) {
@@ -70,6 +77,7 @@ void TodoDB::operator () (Mode mode) {
 	initColourPost();
 	switch (mode) {
 		case Add : triggerEvent("add"); add(); break;
+		case Link : triggerEvent("link"); link(); break;
 		case Remove : triggerEvent("remove"); remove(); break;
 		case View : triggerEvent("view"); view(); break;
 		case Edit : triggerEvent("edit"); edit(); break;
@@ -101,7 +109,7 @@ int looking = destringify<int>(index);
 				try {
 				string ns = index.substr(index.find(".") + 1);
 
-					return find((*i).child, ns);
+					return find(*i->child, ns);
 				} catch (exception &e) {
 					throw exception("couldn't find index '" + index + "'");
 				}
@@ -121,7 +129,7 @@ int looking = destringify<int>(index);
 				try {
 				string ns = index.substr(index.find(".") + 1);
 
-					return findContainer(const_cast<multiset<Todo>& >((*i).child), ns);
+					return findContainer(const_cast<multiset<Todo>& >(*i->child), ns);
 				} catch (exception &e) {
 					throw exception("couldn't find index '" + index + "'");
 				}
@@ -142,12 +150,14 @@ int looking;
 			if (index.find(".") != string::npos) {
 				try {
 				string ss = index.substr(index.find(".") + 1);
-					erase(const_cast<Todo&>((*i)).child, ss);
+					erase(*const_cast<Todo&>(*i).child, ss);
 				} catch (exception &e) {
 					throw exception("couldn't find index '" + index + "'");
 				}
-			} else
+			} else {
 				todo.erase(i);
+				setDirty(true);
+			}
 			return;
 		}
 	throw exception("couldn't find index '" + index + "'");
@@ -158,18 +168,20 @@ string text;
 
 	if (options.text != "") text = options.text;
 	else {
-		if (isatty(1)) {
+		if (isatty(0)) {
 			if (options.verbose)
 				cout << info << "Enter text for the title of this todo list." << normal << endl;
 			text = readText("text> ", titleText);
 		} else {
-		char line[2048];
+		string line;
 
-			while (cin.getline(line, 2048))
+			while (getline(cin, line))
 				text += line + '\n';
 		}
 	}
 	titleText = text;
+
+	setDirty(true);
 }
 
 void TodoDB::parse(vector<XML*>::const_iterator begin, 
@@ -201,7 +213,25 @@ void TodoDB::parse(vector<XML*>::const_iterator begin,
 					todo.text = trim((*x.child().begin())->body());
 					todo.added = destringify<time_t>(attrib["time"]);
 
-					parse(x.child().begin(), x.child().end(), todo.child);
+					parse(x.child().begin(), x.child().end(), *todo.child);
+
+					out.insert(todo);
+				} else if (x.name() == "link") {
+				TodoDB newDb;
+				Todo todo;
+				map<string, string> &attrib = *const_cast<map<string, string>* >(&x.attrib());
+
+					if (attrib.find("filename") == attrib.end())
+						throw exception("require 'filename' attribute for 'link' element");
+
+					newDb.load(attrib["filename"]);
+					
+					if (newDb.titleText == "")
+						todo.text = attrib["filename"];
+					else
+						todo.text = newDb.titleText;
+
+					todo.child = &newDb.todo;
 
 					out.insert(todo);
 				} else
@@ -228,7 +258,7 @@ void TodoDB::fixParents(multiset<Todo> &todo, Todo *parent) {
 
 		if (parent)
 			todo.parent = parent;
-		fixParents(todo.child, &todo);
+		fixParents(*todo.child, &todo);
 	}
 }
 
@@ -236,6 +266,8 @@ void TodoDB::load(string const &file) {
 Loader loader;
 string lastError;
 
+	setDirty(false);
+
 	stat(file.c_str(), &_stat);
 	if (options.timeout && options.mode == View && (time(0) - _stat.st_atime < options.timeoutseconds)) {
 		if (options.verbose)
@@ -243,6 +275,16 @@ string lastError;
 		return;
 	}
 
+	options.timeout = false;
+
+	filename = file;
+
+	/* Get the base name */
+string::size_type pos = filename.rfind("/");
+	
+	if (pos > 0 && pos != string::npos)
+		basepath = filename.substr(0, pos);
+
 ifstream in(file.c_str());
 
 	statSuccessful = false;
@@ -273,39 +315,60 @@ ifstream in(file.c_str());
 }
 
 void TodoDB::save(multiset<Todo> const &todo, ostream &of, int ind) {
+
 	for (multiset<Todo>::const_iterator i = todo.begin(); i != todo.end(); i++)
 	{
-		of	<< string(ind * 4, ' ') << "<note"
-			<< " priority=\"" << symbolisePriority((*i).priority) << "\""
-			<< " time=\"" << (*i).added << "\"";
-		if ((*i).done) {
-			of	<< " done=\"" << (*i).doneTime<< "\"";
+		cerr << "saving: " << i->text << endl;
+		if (i->type == Todo::Link) {
+			of  << string(ind * 4, ' ') << "<link"
+			    << " filename=\"" << i->todofile << "\""
+				<< " priority=\"" << symbolisePriority(i->priority) << "\""
+				<< " time=\"" << i->added << "\""
+				<< "/>" << endl;
+
+			if (i->db)
+				i->db->save(i->todofile);
+
+			/* Restore the TODODB environment variable. */
+string envar = "TODODB=" + filename;
+
+			//setenv("TODODB", options.database.c_str(), 1);
+			putenv(strdup(const_cast<char*>(envar.c_str())));
+
 		}
-		of << ">" << endl;
-		of << string((ind + 1) * 4, ' ');
-		of << htmlify((*i).text) << endl;
-		if ((*i).comment != "") {
-			of << "<comment>" << endl;
-			of << string((ind + 2) * 4, ' ');
-			of << htmlify((*i).comment) << endl;
-			of << "</comment>" << endl;
+		else {
+			of	<< string(ind * 4, ' ') << "<note"
+				<< " priority=\"" << symbolisePriority((*i).priority) << "\""
+				<< " time=\"" << (*i).added << "\"";
+			if ((*i).done) {
+				of	<< " done=\"" << (*i).doneTime<< "\"";
+			}
+			of << ">" << endl;
+			of << string((ind + 1) * 4, ' ');
+			of << htmlify((*i).text) << endl;
+			if ((*i).comment != "") {
+				of << "<comment>" << endl;
+				of << string((ind + 2) * 4, ' ');
+				of << htmlify((*i).comment) << endl;
+				of << "</comment>" << endl;
+			}
+			save(*i->child, of, ind + 1);
+			of << string(ind * 4, ' ');
+			of << "</note>" << endl;
 		}
-		save((*i).child, of, ind + 1);
-		of << string(ind * 4, ' ');
-		of << "</note>" << endl;
 	}
 }
 
 void TodoDB::save(string const &file) {
 	// Do backups
-	if (options.backups) {
+	if (dirty && options.backups) {
 	string newname;
 		for (int i = options.backups - 1; i > 0; i--) {
 			newname = file + "." + stringify(i + 1);
 			if (options.verbose > 1)
 				cout << "todo: renaming " << file << "." << i  << " to " << file << "." << i + 1 << endl;
 			chmod(newname.c_str(), 0600);
-			unlink(newname.c_str());
+			::unlink(newname.c_str());
 			rename((file + "." + stringify(i)).c_str(), newname.c_str());
 			chmod(newname.c_str(), 0400);
 		}
@@ -313,7 +376,7 @@ void TodoDB::save(string const &file) {
 			cout << "todo: renaming " << file << " to " << file << ".1" << endl;
 		newname = file + ".1";
 		chmod(newname.c_str(), 0600);
-		unlink(newname.c_str());
+		::unlink(newname.c_str());
 		rename(file.c_str(), newname.c_str());
 		chmod(newname.c_str(), 0400);
 	}
@@ -321,11 +384,11 @@ void TodoDB::save(string const &file) {
 	Saver saver = getSavers();
 	string lastError;
 
-		if (options.verbose > 1)
+		if (dirty && options.verbose > 1)
 			cout << "todo: saving to database '" << file << "'" << endl;
 
 		for (vector<string>::iterator i = options.loaders.begin(); i != options.loaders.end(); ++i) {
-			if (options.verbose > 1)
+			if (dirty && options.verbose > 1)
 				cout << "todo: trying '" << (*i) << "' saver" << endl;
 			if (saver.find(*i) == saver.end())
 				throw exception("couldn't find saver for '" + *i + "'");
@@ -350,6 +413,8 @@ void TodoDB::save(string const &file) {
 					triggerEvent("save");
 					return;
 				}
+				else if (!dirty)
+					return;
 			} catch (std::exception &e) {
 				lastError = e.what();
 			}
@@ -359,51 +424,24 @@ void TodoDB::save(string const &file) {
 	} else {
 		if (options.verbose > 1)
 			cout << "todo: empty database '" << file << "', unlinking" << endl;
-		unlink(file.c_str());
+		::unlink(file.c_str());
 	}
-
-#if 0
-	if (todo.size() || titleText != "") {
-	ofstream of(file.c_str());
-
-		if (of.bad()) throw exception(("couldn't write to '" + file + "'").c_str());
-		of	<< "<!-- Automagically generated by the ToDo program on " << dateToHuman(getCurrentDate()) << " -->" << endl;
-		of << "<todo version=\"" << VERSION << "\">" << endl;
-		if (titleText != "")
-			of	<< "    <title>" << endl 
-				<< "        " << titleText << endl
-				<< "    </title>" << endl;
-		save(todo, of, 1);
-		of << "</todo>" << endl;
-		of.close();
-
-		// Preserve ownership and mode
-		if (statSuccessful) {
-			if (options.verbose > 1)
-				cout << "todo: preserving attributes" << endl;
-			chmod(file.c_str(), _stat.st_mode);
-			chown(file.c_str(), _stat.st_uid, _stat.st_gid);
-		} else {
-			if (options.paranoid) {
-				if (options.verbose > 1)
-					cout << "todo: paranoia check" << endl;
-				stat(file.c_str(), &_stat);
-				if (_stat.st_mode & 0077)
-					cerr << "todo: warning, created database (" << file << ") has group or world permissions" << endl;
-			}
-		}
-	} else {
-		if (options.verbose > 1)
-			cout << "todo: empty database '" << file << "', unlinking" << endl;
-		unlink(file.c_str());
-	}
-#endif
 }
 
 vector<string> TodoDB::getIndexList(string const &str) {
 vector<string> tmp = split(",", str), out;
 
 	for (vector<string>::iterator i = tmp.begin(); i != tmp.end(); i++) {
+		// wildcard?
+		if ((*i)[(*i).size() - 1] == '*') {
+		string base = (*i).substr(0, (*i).rfind("."));
+		Todo *t = find(todo, base);
+
+
+			if (!t) throw exception("can't expand non-existant wildcard note '" + (*i) + "'");
+			for (multiset<Todo>::iterator j = t->child->begin(); j != t->child->end(); j++)
+				out.push_back(base + stringify((*j).index));
+		} else
 		// is it a range?
 		if ((*i).find('-') != string::npos) {
 		string start = (*i).substr(0, (*i).find('-')), 
@@ -426,19 +464,8 @@ vector<string> tmp = split(",", str), out;
 
 			for (int i = a; i <= b; ++i)
 				out.push_back(base + stringify(i));
-		} else {
-		// wildcard?
-			if ((*i)[(*i).size() - 1] == '*') {
-			string base = (*i).substr(0, (*i).rfind("."));
-			Todo *t = find(todo, base);
-
-
-				if (!t) throw exception("can't expand non-existant wildcard note '" + (*i) + "'");
-				for (multiset<Todo>::iterator j = t->child.begin(); j != t->child.end(); j++)
-					out.push_back(base + stringify((*j).index));
-			} else
-				out.push_back((*i));
-		}
+		} else
+			out.push_back((*i));
 	}
 	return out;
 }
@@ -462,7 +489,7 @@ char *pri[] = {
 
 string priority;
 
-	if (isatty(1)) {
+	if (isatty(0)) {
 		// Display list of priorities if at a tty
 		for (int i = 0; i < 5; i++) {
 			cout << info << i + 1 << ". " << TodoDB::priority[i] << pri[i] << normal << "   ";
@@ -471,7 +498,7 @@ string priority;
 		cout << endl;
 	}
 	while (true) {
-		if (isatty(1)) {
+		if (isatty(0)) {
 			if (options.verbose)
 				cout << info << "Enter a priority from those listed above." << normal << endl;
 			priority = readText("priority> ", current, true);
@@ -509,12 +536,11 @@ Todo *parent = 0;
 		text = options.text;
 	else
 	{
-		if (isatty(1)) {
+		if (isatty(0)) {
 			if (options.verbose)
 				cout << info << "Enter text for the item you are adding." << normal << endl;
 			text = readText("text> ", text);
-		} else
-		{
+		} else {
 		string line;
 
 			while (getline(cin, line))
@@ -526,7 +552,13 @@ Todo t;
 
 	t.text = text;
 
-	if (options.priority != -1)
+	if (options.priority == Todo::Default)
+		if (parent)
+			options.priority = parent->priority;
+		else
+			options.priority = Todo::Medium;
+
+	if (options.priority != Todo::None)
 		t.priority = options.priority;
 	else {
 	string defpri;
@@ -542,12 +574,13 @@ Todo t;
 	if (parent) {
 		if (options.verbose > 1)
 			cout << "todo: grafting new item to item " << options.index[0] << endl;
-		parent->child.insert(t);
+		parent->child->insert(t);
+		parent->db->setDirty(true);
 
 		if (options.verbose) {
 		unsigned findindex = 1;
 
-			for (multiset<Todo>::iterator i = parent->child.begin(); i != parent->child.end(); ++i, findindex++)
+			for (multiset<Todo>::iterator i = parent->child->begin(); i != parent->child->end(); ++i, findindex++)
 				if (t == *i)
 					cout << info << "Index of new item is " << options.index[0] << "." << findindex << normal << endl;
 		}
@@ -555,6 +588,7 @@ Todo t;
 		if (options.verbose > 1)
 			cout << "todo: adding new item" << endl;
 		todo.insert(t);
+		setDirty(true);
 
 		if (options.verbose) {
 		unsigned findindex = 1;
@@ -563,6 +597,102 @@ Todo t;
 					cout << info << "Index of new item is " << findindex << normal << endl;
 		}
 	}
+
+}
+
+void TodoDB::link() {
+string filename;
+Todo *parent = 0;
+
+	// grafting, get parents priority and use it as default for children
+	if (options.index.size()) {
+		parent = find(todo, options.index[0]);
+
+		if (!parent) throw exception("couldn't find '" + options.index[0] + "' to graft to");
+	}
+
+	if (options.filename != "")
+		filename = options.filename;
+	else
+	{
+		if (isatty(0)) {
+			if (options.verbose)
+				cout << info << "Enter the filename of the todo database to link." << normal << endl;
+
+			filename = readText("filename> ", filename);
+		} else
+		{
+		string line;
+
+			// XXX
+			while (getline(cin, line))
+				filename += line + '\n';
+		}
+	}
+
+Todo t;
+
+	t.type = Todo::Link;
+
+	if (parent)
+		t.todofile = fixRelativePath(parent->db->basepath, filename);
+	else
+		t.todofile = filename;
+
+ifstream in(t.todofile.c_str());
+	if (!in) throw exception("could not link to '" + t.todofile + "', unreadable/not found");
+
+	if (options.priority == Todo::Default)
+		options.priority = Todo::Medium;
+
+	if (options.priority != Todo::None)
+		t.priority = options.priority;
+	else {
+		string defpri;
+
+		if (parent)
+			defpri = symbolisePriority(parent->priority);
+		else
+			defpri = "medium";
+
+		t.priority = getPriority(defpri);
+	}
+
+	t.added = getCurrentDate();
+
+	// grafting
+	if (parent) {
+		if (options.verbose > 1)
+			cout << "todo: grafting new item to item " << options.index[0] << endl;
+
+		parent->child->insert(t);
+		parent->db->setDirty(true);
+
+		if (options.verbose) {
+			unsigned findindex = 1;
+
+				for (multiset<Todo>::iterator i = parent->child->begin(); i != parent->child->end(); ++i, findindex++)
+					if (t == *i)
+						cout << info << "Index of new item is " << options.index[0] << "." << findindex << normal << endl;
+		}
+	} else {
+		if (options.verbose > 1)
+			cout << "todo: adding new item" << endl;
+
+		t.todofile = filename;
+
+		todo.insert(t);
+
+		setDirty(true);
+
+		if (options.verbose) {
+		unsigned findindex = 1;
+
+			for (multiset<Todo>::iterator i = todo.begin(); i != todo.end(); ++i, findindex++)
+				if (t == *i)
+					cout << info << "Index of new item is " << findindex << normal << endl;
+		}
+	}
 }
 
 void TodoDB::edit() {
@@ -570,14 +700,19 @@ void TodoDB::edit() {
 string const &index = options.index[0];
 Todo t = *find(todo, index);
 
+	if (t.type == Todo::Link)
+		throw exception("can't edit the body of a link");
 	erase(todo, options.index[0]);
 	if (t.done) throw exception("you can't edit an item that is done");
 	if (options.verbose)
 		cout << info << "Modify the text of the item you are editing." << normal << endl;
 	t.text = readText("text> ", t.text);
-	if (options.priority != -1)
+
+	if (options.priority != Todo::None && options.priority != Todo::Default)
 		t.priority = options.priority;
-	t.priority = getPriority(symbolisePriority(t.priority));
+
+	if (options.priority != Todo::Default)
+		t.priority = getPriority(symbolisePriority(t.priority));
 
 	if (options.comment || t.comment != "") {
 		if (options.verbose)
@@ -590,9 +725,13 @@ string parent = index.substr(0, index.rfind("."));
 
 	Todo *p = find(todo, parent);
 
-		p->child.insert(t);
-	} else
+		p->child->insert(t);
+		p->db->setDirty(true);
+	} else {
+		t.db = this;
 		todo.insert(t);
+		setDirty(true);
+	}
 }
 
 void TodoDB::remove() {
@@ -603,12 +742,14 @@ int erased = 0;
 	for (vector<string>::iterator j = remove.begin(); j != remove.end(); j++) {
 	Todo *t = find(todo, *j);
 
-		if (t)
-		{
+		if (t) {
+			if (t->type == Todo::Link && options.verbose)
+				cout << info << "todo: removing link to database '" << t->todofile << "'" << normal << endl;
 			erased++;
 			if (options.verbose > 1)
-				cout << "todo: permanently removing item '" << (*j) << "'" << endl;
+				cout << info << "todo: permanently removing item '" << (*j) << "'" << normal << endl;
 			erase(todo, *j);
+			t->db->setDirty(true);
 		} else
 			notfound.push_back(*j);
 	}
@@ -618,12 +759,31 @@ int erased = 0;
 		cout << info << "todo: erased " << erased << " records" << normal << endl;
 }
 
-void TodoDB::filterChildren(Todo &todo, bool filter) {
-	todo.filtered = filter;
+void TodoDB::filterChildren(Todo &todo, FilterChildren filter) {
+	todo.filtered = filter == FILTERED ? true : false;
 	if (!todo.filtered && todo.parent) 
 		todo.incUnFilteredChildren();
-	for (multiset<Todo>::iterator i = todo.child.begin(); i != todo.child.end(); i++)
-		filterChildren((Todo&)*i, filter);
+	if (filter == NOTFILTERED)
+		for (multiset<Todo>::iterator i = todo.child->begin(); i != todo.child->end(); i++)
+			filterChildren((Todo&)*i, NOTFILTERED);
+	else
+	if (filter == CHILDRENNOTFILTERED)
+		for (multiset<Todo>::iterator i = todo.child->begin(); i != todo.child->end(); i++) {
+			if ((
+				// ugh
+				options.filter.done && (
+					(options.filter.donedir == Options::Positive) ||
+					(options.filter.donedir == Options::Equal && i->done) ||
+					(options.filter.donedir == Options::Negative && !i->done)
+					)
+				) || 
+				(!options.filter.done && !i->done)
+				) {
+				// doesn't change sort order
+				const_cast<Todo*>(&*i)->filtered = false;
+				const_cast<Todo*>(&*i)->incUnFilteredChildren();
+			}
+		}
 }
 
 void TodoDB::filterView(multiset<Todo> &todo) {
@@ -638,7 +798,7 @@ int n = 1;
 			skipchildren = true;
 		else
 		if (filter.show == Options::Positive)
-			filterChildren(todo, false);
+			filterChildren(todo, NOTFILTERED);
 		else {
 			// Filter on done
 			if (filter.done) {
@@ -673,7 +833,7 @@ int n = 1;
 				if (filter.childrendir == Options::Negative) {
 				bool state = todo.filtered;
 
-					filterChildren(todo, true);
+					filterChildren(todo, FILTERED);
 					todo.filtered = state;
 					skipchildren = true;
 				}
@@ -688,7 +848,7 @@ int n = 1;
 			todo.incUnFilteredChildren();
 
 		//if (!todo.filterchildren)
-		if (!skipchildren) filterView(todo.child);
+		if (!skipchildren) filterView(*todo.child);
 	}
 }
 
@@ -703,14 +863,12 @@ void TodoDB::filterView() {
 	Todo *t = find(todo, (*i).first);
 
 		if ((*i).second == Options::Negative)
-			filterChildren(*t, true);
+			filterChildren(*t, FILTERED);
 		else
 		if ((*i).second == Options::Positive)
-			filterChildren(*t, false);
-		else {
-			t->filtered = false;
-			t->incUnFilteredChildren();
-		}
+			filterChildren(*t, NOTFILTERED);
+		else
+			filterChildren(*t, CHILDRENNOTFILTERED);
 	}
 }
 
@@ -729,10 +887,18 @@ void TodoDB::view(multiset<Todo> const &todo, int ind) {
 
 				cout << comment << string(indent, ' ');
 				if (options.summary) {
-					if ((int)todo.comment.size() > options.columns - 3 - indent)
-						cout << "(" << todo.comment.substr(0, options.columns - 3 - indent) << ")" << info << "+";
+				const string s = todo.comment;
+
+					if (s.find('\n') != string::npos) {
+						if ((int)s.find('\n') < options.columns - 1 - indent)
+							cout << s.substr(0, s.find('\n')) << info << "+" << normal;
+						else
+							cout << s.substr(0, options.columns - 1 - indent) << info << "+" << normal;
+					} else
+					if ((int)s.size() > options.columns - 3 - indent)
+						cout << "(" << s.substr(0, options.columns - 3 - indent) << ")" << info << "+";
 					else
-						cout << "(" << todo.comment << ")";
+						cout << "(" << s << ")";
 				} else
 					wraptext(cout, "(" + todo.comment + ")", indent, indent, options.columns);
 				cout << normal << endl;
@@ -742,7 +908,7 @@ void TodoDB::view(multiset<Todo> const &todo, int ind) {
 		//if (todo.filterchildren && !todo.unfilteredchildren) continue;
 		if (todo.filtered && todo.unfilteredchildren)
 			cout << string(4 * ind, ' ') << info << todo.index << "..." << normal << endl;
-		view(todo.child, ind + 1);
+		view(*todo.child, ind + 1);
 	}
 }
 
@@ -771,7 +937,7 @@ void TodoDB::generate(ostream &out, multiset<Todo> const &todo, int ind) {
 		//if (todo.filterchildren) continue;
 		if (todo.filtered && todo.unfilteredchildren)
 			out << string(4 * ind, ' ') << info << todo.index << "..." << normal << endl;
-		generate(out, todo.child, ind + 1);
+		generate(out, *todo.child, ind + 1);
 	}
 }
 
@@ -805,7 +971,7 @@ int TodoDB::markDone(Todo &todo) {
 int count = 1;
 
 	todo.done = true;
-	for (multiset<Todo>::iterator i = todo.child.begin(); i != todo.child.end(); i++)
+	for (multiset<Todo>::iterator i = todo.child->begin(); i != todo.child->end(); i++)
 		count += markDone(const_cast<Todo&>(*i));
 	return count;
 }
@@ -825,6 +991,7 @@ int marked = 0;
 			if (options.verbose)
 				cout << info << "Enter comment for this item." << normal << endl;
 			t->comment = readText("comment> ", t->comment);
+			t->db->setDirty(true);
 		} else
 			notfound.push_back(*j);
 	}
@@ -846,6 +1013,8 @@ int marked = 0;
 			marked++;
 			if (options.verbose > 1)
 				cout << "todo: marked '" << *j << "' as not done" << endl;
+
+			t->db->setDirty(true);
 		} else
 			notfound.push_back(*j);
 	}
@@ -897,21 +1066,32 @@ multiset<Todo> *ap = 0;
 int delim = aps.rfind('.');
 	if (delim != -1) {
 		aps = aps.substr(0, delim);
-		ap = &(find(aps)->child);
+		ap = &(*find(aps)->child);
 	} else
 		ap = &todo;
 
 Todo tmp = *a;
 	for (multiset<Todo>::iterator i = ap->begin(); i != ap->end(); ++i)
 		if (&(*i) == a) {
+			i->db->setDirty(true);
 			ap->erase(i);
 			break;
 		}
 
-	if (options.index.size() > 1)
-		find(options.index[1])->child.insert(tmp);
-	else
+	if (options.index.size() > 1) {
+		Todo *t = find(options.index[1]);
+		tmp.parent = t;
+		tmp.db = t->db;
+		t->child->insert(tmp);
+		t->db->setDirty(true);
+	} else {
+		tmp.db = this;
 		todo.insert(tmp);
+		setDirty(true);
+	}
+	
+
+//	tmp.child = NULL;
 }
 
 ostream &TodoDB::StreamColour::veryhigh(ostream &os) { return os << normal << streamColour["veryhigh"].attribute << streamColour["veryhigh"].colour; }
@@ -1001,20 +1181,41 @@ Options::Dir dir = Options::Equal;
 							i += dir * multiplier;
 						i *= defaultindent;
 						if (options.summary) {
-							if ((int)item.text.size() > options.columns - 1 - indent)
-								out << item.text.substr(0, options.columns - 1 - indent) << info << "+" << normal;
+						const string s = item.text;
+
+							if (s.find('\n') != string::npos) {
+								if ((int)s.find('\n') < options.columns - 1 - indent)
+									out << s.substr(0, s.find('\n')) << info << "+" << normal;
+								else
+									out << s.substr(0, options.columns - 1 - indent) << info << "+" << normal;
+							} else
+							if ((int)s.size() > options.columns - 1 - indent)
+								out << s.substr(0, options.columns - 1 - indent) << info << "+" << normal;
 							else
-								out << item.text;
+								out << s;
 						} else
 							wraptext(out, item.text, i, indent, options.columns);
 					}
 					break;
 					case 't' :
 						if (options.summary) {
-							if ((int)item.text.size() > options.columns - 1 - indent)
+/*							if ((int)item.text.size() > options.columns - 1 - indent)
 								out << item.text.substr(0, options.columns - 1 - indent) << info << "+" << normal << endl;
 							else
 								out << item.text << endl;
+							indent = 0;*/
+						const string s = item.text;
+
+							if (s.find('\n') != string::npos) {
+								if ((int)s.find('\n') < options.columns - 1 - indent)
+									out << s.substr(0, s.find('\n')) << info << "+" << normal << endl;
+								else
+									out << s.substr(0, options.columns - 1 - indent) << info << "+" << normal << endl;
+							} else
+							if ((int)s.size() > options.columns - 1 - indent)
+								out << s.substr(0, options.columns - 1 - indent) << info << "+" << normal << endl;
+							else
+								out << s << endl;
 							indent = 0;
 						} else {
 							out << item.text;
@@ -1044,7 +1245,7 @@ Options::Dir dir = Options::Equal;
 					string s;
 					
 						if (item.done)
-							s = dateToHuman(item.doneTime);
+							s = "completed on " + dateToHuman(item.doneTime);
 						else
 							s = "incomplete";
 						out << s;
@@ -1053,7 +1254,7 @@ Options::Dir dir = Options::Equal;
 					break;
 					case 'D' :
 						if (item.done) {
-						string s = elapsedToHuman(item.added, item.doneTime);
+						string s = elapsedToHuman(item.added, item.doneTime) + " elapsed";
 							out << s;
 							indent += s.size();
 						} else {
@@ -1075,7 +1276,7 @@ Options::Dir dir = Options::Equal;
 					string s;
 					bool filteredchildren = false;
 
-						for (multiset<Todo>::const_iterator i = item.child.begin(); i != item.child.end(); i++)
+						for (multiset<Todo>::const_iterator i = item.child->begin(); i != item.child->end(); i++)
 							if ((*i).filtered) {
 								filteredchildren = true;
 								break;
@@ -1098,7 +1299,7 @@ Options::Dir dir = Options::Equal;
 					case 'f' : {
 					bool filteredchildren = false;
 
-						for (multiset<Todo>::const_iterator i = item.child.begin(); i != item.child.end(); i++)
+						for (multiset<Todo>::const_iterator i = item.child->begin(); i != item.child->end(); i++)
 							if ((*i).filtered) {
 								filteredchildren = true;
 								break;
@@ -1171,8 +1372,125 @@ char const *argv[v.size() + 1];
 		cout << "todo: event '" << event << "' triggered ('" << str::join(" ", options.event[event]) << "')" << endl;
 	argv[0] = "todo";
 	for (unsigned i = 0; i < v.size(); i++) {
-		v[i] = expandEnvars(v[i]);
-		argv[i + 1] = v[i].c_str();
+		string s = expandEnvars(v[i]);
+		argv[i + 1] = strdup(s.c_str());
 	}
 	parseArgs(*this, v.size() + 1, argv);
 }
+
+string TodoDB::fixPath(string path)
+{
+	string newPath;
+	string::iterator iter;
+	
+	for (iter = path.begin(); iter != path.end(); iter++)
+	{
+		if (*iter == '.')
+		{
+			if (*(iter + 1) == '.' && *(iter + 2) == '/')
+			{
+				if (!newPath.empty())
+				{
+					newPath.erase(newPath.rfind("/", newPath.size() - 2) + 1);
+				}
+
+				iter += 2;
+
+				continue;
+			}
+			else if (*(iter + 1) == '/')
+			{
+				iter++;
+
+				continue;
+			}
+			else if (iter + 1 == path.end())
+				continue;
+		}
+		else if (*iter == '/')
+		{
+			while (*(iter + 1) == '/')
+				iter++;
+		}
+
+		newPath.push_back(*iter);
+	}
+
+	return newPath;
+}
+
+string TodoDB::fixRelativePath(string base, string path)
+{
+	char cwd[BUFSIZ];
+	string relBase, relDestPath;
+	string prefix;
+	string result;
+	size_t s;
+
+	if (path[0] == '/')
+		return path;
+
+	/* Chop off any trailing '/''s */
+	if ((s = base.find_last_not_of('/')) != string::npos)
+		base.erase(s + 1);
+
+	if ((s = path.find_last_not_of('/')) != string::npos)
+		path.erase(s + 1);
+
+	/* Get the working directory. */
+	getcwd(cwd, BUFSIZ);
+
+	/* Build the paths appropriately. */
+	if (base[0] != '/')
+	{
+		relBase = cwd;
+		relBase += "/" + base;
+	}
+	else
+		relBase = base;
+
+	relDestPath = cwd;
+	relDestPath += "/" + path;
+
+	/* Fix up the paths to process stuff like '..' */
+	relBase = fixPath(relBase);
+	relDestPath = fixPath(relDestPath);
+
+	/* See how much of the prefix we can chop off. */
+	s = 0;
+
+	do
+	{
+		size_t s1 = relBase.find("/", s);
+
+		if (s1 == 0 && relBase[0] != '/')
+			break;
+
+		if (relBase.substr(0, s1) == relDestPath.substr(0, s1))
+			s = s1 + 1;
+		else
+			prefix = relBase.substr(0, s);
+
+	} while (prefix.empty());
+
+	relBase.erase(0, prefix.size());
+	relDestPath.erase(0, prefix.size());
+	
+	if (relBase.empty())
+		result = relDestPath;
+	else
+	{
+		/* Look at the number of '/''s in relBase */
+		size_t count = 0;
+
+		for (s = 0; (s = relBase.find("/", s + 1)) != string::npos; count++)
+			;
+
+		for (size_t i = 0; i <= count; i++)
+			result += "../";
+
+		result += relDestPath;
+	}
+
+	return result;
+}
