@@ -29,6 +29,7 @@ class TodoDB {
 
 		enum Mode {
 			Add,
+			Link,
 			Remove,
 			View,
 			Edit,
@@ -57,14 +58,19 @@ class TodoDB {
 		void setColour(string const &item, string const &colour);
 
 		void setDirty(bool d) { dirty = d; }
+		bool isDirty() const { return dirty; }
 
 		void triggerEvent(string const &event);
 
 		multiset<Todo> todo;
 		string titleText;
+		string filename, basepath;
 
 	private :
+		enum FilterChildren { FILTERED, NOTFILTERED, CHILDRENNOTFILTERED };
+
 		void add();
+		void link();
 		void remove();
 		void view();
 		void edit();
@@ -87,7 +93,7 @@ class TodoDB {
 		void parse(vector<XML*>::const_iterator begin,
 		    vector<XML*>::const_iterator end, multiset<Todo> &out);
 		void save(multiset<Todo> const &todo, ostream &of, int ind);
-		void filterChildren(Todo &todo, bool filter = true);
+		void filterChildren(Todo &todo, FilterChildren filter = FILTERED);
 		void filterView();
 		void filterView(multiset<Todo> &todo);
 		void stats(multiset<Todo> const &todo, int ind);
@@ -103,6 +109,9 @@ class TodoDB {
 
 		void formatItem(ostream &out, int depth, Todo const &item, string const &format);
 
+		string fixPath(string path);
+		string fixRelativePath(string base, string path);
+
 		struct stat _stat;
 		bool statSuccessful;
 		bool dirty;
