@@ -30,7 +30,7 @@ Options::Options() :
 	format["display"] = "%4>%i%f%[info]%2n.%[priority]%+1T\n";
 	format["comment-display"] = "%4>%i%f%[info]%2n.%[priority]%+1T\n%+1i%[comment](Comment: %C)\n";
 	format["verbose-display"] = "%i%[info]%f%2n.%[priority]%+1T\n%+1i%[info]Added: %[normal]%c  %[info]Completed: %[normal]%d\n%+1i%[info]Duration: %[normal]%D  %[info]Priority: %[normal]%p\n\n";
-	format["generated"] = "%2>%i-%+1T\n%+1i(added %c, completed %d, priority %p)\n\n";
+	format["generated"] = "%2>%i-%+1T\n%+1i(added %c, %d, priority %p)\n\n";
 	format["verbose-generated"] = "%2>%i- %+1T\n\n%+1iAdded: %c  Completed: %d\n%+1iDuration: %D  Priority: %p\n\n\n";
 
 	loaders.push_back("xml");
@@ -41,6 +41,8 @@ Options::Options() :
 	else
 		globaldatabase = "";
 
+	filename = "";
+
 	columns = getWidth();
 }
 
@@ -191,7 +193,10 @@ string myname = argv[0];
 	if (myname.rfind("tdd") == myname.size() - 3)
 		options.mode = TodoDB::Done;
 
-enum { Help = -100, Remove, Version, Title, Colour, Mono, 
+	if (myname.rfind("tdl") == myname.size() - 3)
+		options.mode = TodoDB::Link;
+
+enum { Help = -100, Remove, Version, Title, Colour, Mono, ForceColour,
 	Database, GlobalDatabase, DateFormat, DisplayFormat,
 	VerboseDisplayFormat, VerboseGeneratedFormat,
 	GeneratedFormat, Sort, Paranoid, DatabaseLoaders, Backup,
@@ -199,12 +204,14 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 
 	args.addArgument('a', "add", CommandArgs::Optional);
 	args.setHelp('a', "Add a note (will prompt for a note if one is not supplied).");
+	args.addArgument('l', "link", CommandArgs::Required);
+	args.setHelp('l', "Links another todo file to this one.");
 	args.addArgument('g', "graft", CommandArgs::Required);
-	args.setHelp('g', "In conjunction with --add, graft the new item to the specified item.");
+	args.setHelp('g', "In conjunction with --add or --link, graft the new item to the specified item.");
 	args.addArgument('R', "reparent", CommandArgs::Required);
 	args.setHelp('R', "Change the parent of the first item index to the second item index (seperated by a comma). If no second index is given the item is reparented to the root of the tree. eg. todo -R 1,2");
 	args.addArgument('p', "priority", CommandArgs::Required);
-	args.setHelp('p', "In conjunction with --add, set the default priority (veryhigh|high|medium|low|verylow).");
+	args.setHelp('p', "In conjunction with --add, set the default priority (default|veryhigh|high|medium|low|verylow).");
 	args.addArgument('e', "edit", CommandArgs::Required);
 	args.setHelp('e', "Edit the note that is indexed by the given number.");
 	args.addArgument(Remove, "remove", CommandArgs::Required);
@@ -233,6 +240,8 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 	args.setHelp(Colour, "Override default colours of todo items. Multiple colours can be specified, seperated by commas. eg. high=red,medium=white. Items are bolded by prefixing them with a '+'.");
 	args.addArgument(Mono, "mono");
 	args.setHelp(Mono, "Disable all use of colour.");
+	args.addArgument(ForceColour, "force-colour");
+	args.setHelp(ForceColour, "Force use of colour.");
 	args.addArgument(Help, "help");
 	args.setHelp(Help, "Display this help.");
 	args.addArgument(Version, "version");
@@ -278,10 +287,11 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 					<< "Where <arguments> can be any of the following:" << endl;
 				args.displayHelp(cout, options.columns);
 				cout << endl
-					<< "In addition, there are four convenience symlinks. These are 'tda', 'tdr'," << endl
-					<< "'tdd' and 'tde'. For 'tde', 'tdd' and 'tdr' supply an index to edit, mark" << endl
-					<< "done and remove respectively. For 'tda' supply the text of the todo item and" << endl
-					<< "optionally the priority."
+					<< "In addition, there are five convenience symlinks. These are 'tda', 'tdr'," << endl
+					<< "'tdd', 'tde', and 'tdl'. For 'tde', 'tdd' and 'tdr' supply an index to edit," << endl
+					<< "mark done and remove respectively. For 'tda' supply the text of the todo item" << endl
+					<< "item and optionally the priority. For 'tdl' supply the path to another todo" << endl
+					<< "file to link in to the current todo file." << endl
 					<< "eg. tde 1" << endl;
 				exit(0);
 			break;
@@ -355,6 +365,9 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 				}
 			}
 			break;
+			case ForceColour :
+				term::forceColour(true);
+			break;
 			case Format : {
 			vector<string> comp;
 			
@@ -495,6 +508,10 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 				if (i.parameter()) options.text = i.parameter();
 				options.mode = TodoDB::Add;
 			break;
+			case 'l' :
+				if (i.parameter()) options.filename = i.parameter();
+				options.mode = TodoDB::Link;
+			break;
 			case 'g' : 
 				options.index = todo.getIndexList(i.parameter());
 				if (options.index.size() > 1)
@@ -545,7 +562,10 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 			case 0 :
 				// default parameter for tda is the todo text body
 				if (options.mode == TodoDB::Add) {
-					options.text = i.parameter();
+					if (options.text != "")
+						options.text += " " + string(i.parameter());
+					else
+						options.text = i.parameter();
 					continue;
 				}
 
@@ -567,6 +587,12 @@ enum { Help = -100, Remove, Version, Title, Colour, Mono,
 					continue;
 				}
 
+				// default parameter for tdl is the path to a todo file
+				if (options.mode == TodoDB::Link) {
+					options.filename = i.parameter();
+					continue;
+				}
+
 				// default parameters for viewing are filters
 				if (options.mode == TodoDB::View) {
 					try {
@@ -585,14 +611,16 @@ string symbolisePriority(string sym) {
 static struct {
 	char const *s, *n;
 } s[] = {
+	{ "default", "-2" },
 	{ "verylow", "5" },
 	{ "low", "4" },
 	{ "medium", "3" },
 	{ "high", "2" },
 	{ "veryhigh", "1" },
+	{ 0, 0 },
 };
 
-	for (int i = 0; i < 5; i++)
+	for (int i = 0; s[i].s; i++)
 		if (sym == s[i].s || sym == s[i].n) return s[i].s;
 	throw logic_error("unknown priority level '" + sym + "'");
 }
@@ -602,14 +630,16 @@ static struct {
 	char const *s;
 	Todo::Priority p;
 } s[] = {
+	{ "default", Todo::Default },
 	{ "verylow", Todo::VeryLow },
 	{ "low", Todo::Low },
 	{ "medium", Todo::Medium },
 	{ "high", Todo::High },
 	{ "veryhigh", Todo::VeryHigh },
+	{ 0, Todo::None },
 };
 
-	for (int i = 0; i < 5; i++)
+	for (int i = 0; s[i].s; i++)
 		if (sym == s[i].p)
 			return s[i].s;
 	throw logic_error("unknown priority level '" + stringify<int>(sym) + "'");
@@ -620,15 +650,17 @@ static struct {
 	char const *s, *n;
 	Todo::Priority p;
 } s[] = {
+	{ "default", "-2", Todo::Default },
 	{ "verylow", "5", Todo::VeryLow },
 	{ "low", "4", Todo::Low },
 	{ "medium", "3", Todo::Medium },
 	{ "high", "2", Todo::High },
 	{ "veryhigh", "1", Todo::VeryHigh },
+	{ 0, 0, Todo::None },
 };
 
 	sym = trim(sym);
-	for (int i = 0; i < 5; i++)
+	for (int i = 0; s[i].s; i++)
 		if (sym == s[i].s || sym == s[i].n)
 			return s[i].p;
 	throw logic_error("unknown symbolic priority level '" + sym + "'");
@@ -693,6 +725,7 @@ static int init_rl() {
 
 static int nuke_on_hit() {
 int c = rl_getc(stdin);
+
 	if (nukeonhit && c != 13) {
 		rl_delete_text(0, rl_end);
 		rl_end = rl_mark = rl_point = 0;
@@ -700,6 +733,10 @@ int c = rl_getc(stdin);
 		rl_clear_message();
 		nukeonhit = false;
 	}
+	if (c == 14) {
+		rl_insert_text("\n");
+		return 0;
+	}
 	return c;
 }
 
@@ -711,11 +748,6 @@ string out;
 
 	nukeonhit = nuke;
 
-#if 0
-	cout << prompt;
-	getline(cin, out);
-#endif
-
 	rl_buffer = &existing;
 char const *tmp = readline(const_cast<char*>(prompt.c_str()));
 	if (tmp) out = tmp;
@@ -798,7 +830,7 @@ vector<string> rcfile, out;
 						event != "edit" && event != "generate" &&
 						event != "done" && event != "done" && event != "notdone" &&
 						event != "title" && event != "reparent" && event != "create" &&
-						event != "save" && event != "load")
+						event != "save" && event != "load" && event != "link")
 						throw runtime_error(string("unknown event '") + event + "'");
 
 					is >> str;
@@ -847,10 +879,6 @@ vector<string> rcfile, out;
 
 string expandEnvars(string const &str) {
 string out;
-string envar = "TODODB=" + options.database;
-
-	//setenv("TODODB", options.database.c_str(), 1);
-	putenv(const_cast<char*>(envar.c_str()));
 
 	for (unsigned i = 0; i < str.size(); ++i)
 		if (str[i] == '$') {
@@ -861,8 +889,8 @@ string envar = "TODODB=" + options.database;
 			if (i != mark) {
 			string var = str.substr(mark, i - mark);
 
-				if (!getenv(var.c_str()))
-					throw runtime_error("no such environment variable '" + var + "'");
+				if (!getenv(var.c_str()) && options.verbose)
+					cout << "notice: no such environment variable '" + var + "'." << endl;
 				out += getenv(var.c_str());
 			} else
 				out += '$';
