@@ -17,7 +17,7 @@ struct Options {
 
 	int verbose;
 	bool mono, paranoid, global, summary, timeout, comment;
-	string text, database, globaldatabase, 
+	string text, database, globaldatabase, filename,
 		dateformat;
 	map<string, string> format;
 
